package pomodoro

import (
	"github.com/gorilla/websocket"
	"sync"
)

const (
	workDuration       = 15 * 60
	shortBreakDuration = 5 * 60
	longBreakDuration  = 10 * 60
	sessions           = 4
)

var pomodoroRunning bool
var pomodoroPaused bool

var pomodoroStopChannel = make(chan bool, 1)
var pomodoroPauseChannel = make(chan bool, 1)
var pomodoroResumeChannel = make(chan bool, 1)

var mu sync.Mutex // to synchronize access to shared state

// RunPomodoroTimer iterates the Pomodoro work/break sessions.
func RunPomodoroTimer(clients map[*websocket.Conn]bool) {
	mu.Lock()
	pomodoroRunning = true
	pomodoroPaused = false
	mu.Unlock()

	for session := 1; session <= sessions; session++ {
		if !startTimer(clients, workDuration, "Work", session) {
			break
		}
		if session == sessions {
			if !startTimer(clients, longBreakDuration, "LongBreak", session) {
				break
			}
		} else {
			if !startTimer(clients, shortBreakDuration, "ShortBreak", session) {
				break
			}
		}
	}

	mu.Lock()
	pomodoroRunning = false
	mu.Unlock()
}

// StopPomodoro sends a signal to stop the running Pomodoro timer.
func StopPomodoro() {
	pomodoroStopChannel <- true
}

func PausePomodoro() {
	pomodoroPauseChannel <- true
}

func ResumePomodoro() {
	pomodoroResumeChannel <- true
}

func IsPomodoroRunning() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroRunning
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroPaused
}
